package abstractFactory.hk;

import javax.swing.*;
import java.awt.event.*;

/**
 * Implements the driver for the Abstract Factory design pattern example.
 * 
 * Intent: Provide an interface for creating families of related or dependent
 * objects without specifying their concrete classes.
 * 
 * As an example scenario, our abstract factory team defines two unbound roles
 * each of which used to create related products (Swing GUI elements).
 * 
 * The driver is a swing GUI that allows the user to choose between the two
 * concrete factory teams RegularFactoryTeam and FramedFactoryTeam,
 * and creates a new GUI with elements from the respective factory.
 * 
 * RegularFactoryTeam declares roles that create standard Swing GUI
 * elements, while FramedFactoryTeam declares roles that create
 * elements which are framed.
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {

	/**
	 * A concrete factory team that creates regular GUI components
	 */
	final private static ComponentFactoryTeam factory1 = new RegularFactoryTeam();

	/**
	 * A concrete factory team that creates framed GUI components
	 */
	final private static ComponentFactoryTeam factory2 = new FramedFactoryTeam();

	/**
	 * Stores the currently selected factory
	 */
	private static ComponentFactoryTeam factory = factory1;

	/**
	 * Creates the initial GUI that allows the user to choose a factory and
	 * generate a new GUI with the elements that the respective factory
	 * provides.
	 * 
	 * @return a JPanel containing the GUI
	 */
	private static JPanel createGUI() {
		ActionListener radioListener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (e.getActionCommand().equals("factory1"))
					factory = factory1;
				else
					factory = factory2;
			}
		};

		JPanel panel = new JPanel();
		JRadioButton factoryButton1 = new JRadioButton("use Factory 1");
		JRadioButton factoryButton2 = new JRadioButton("use Factory 2");
		factoryButton1.setActionCommand("factory1");
		factoryButton2.setActionCommand("factory2");
		factoryButton1.addActionListener(radioListener);
		factoryButton2.addActionListener(radioListener);
		JButton create = new JButton("Create GUI");

		ButtonGroup choices = new ButtonGroup();

		choices.add(factoryButton1);
		choices.add(factoryButton2);

		create.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				/*Display display =*/ new Display(factory);
			}
		});
		panel.add(factoryButton1);
		panel.add(factoryButton2);
		panel.add(create);

		return panel;
	}

	/**
	 * Implements the driver for this design pattern example. It sets up the
	 * initial GUI.
	 */
	public static void main(String[] args) {
		JFrame frame = new JFrame("Abstract Factory OTJ Demo");
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		frame.getContentPane().add(createGUI());
		frame.pack();
		frame.setVisible(true);
	}
}
