package chain_of_responsibility;

import java.util.WeakHashMap;

/**
 * Defines the general behavior of the chain of responsibility design pattern.
 *
 * Each concrete extension of this team defines one particular instance of
 * the pattern, with an arbitrary number of Handler classes involved.
 *
 * The concrete teams defines three things: 
 *    what types can be Handlers
 *    what types can be Requests
 *    what triggers a Request
 *    how each Handler treats a particular request.
 *
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public abstract team class ChainOfResponsibilityProtocol {

    /**
     * This role is used by concrete teams extending this one, to define which base classes
     * handle requests. It models the Handler role.
     */
	protected abstract class Handler {
		/**
		 * Returns true if a Handler wants to handle 
	     * that request. By default, requests are rejected.
	     *
	     * @param request the request to be handled
	     */
		public boolean acceptRequest(Request request) {  
		    return false;
	    }
		
		 /**
	     * Abstract method to handle a request. To be implemented in 
	     * concrete teams or delegated to the base class playing the Handler role.
	     *  
	     * @param request the request to be handled
	     */
		public abstract void handleRequest(Request request);
	}

	/**
	 * The Request role.
	 * 
	 * This abstract role is to be bound to base classes in concrete teams,
	 * which will represent requests. 
	 */
	protected abstract class Request {}

    /**
     * Stores the mapping between Handlers and its Successor.
     * For each handler, its Successor is stored.
     */
	private WeakHashMap<Handler, Handler> successors =
		new WeakHashMap<Handler, Handler>();

    /**
     * Implements the abstracted CoR behavior. Assumption: If the request is
     * not handled, the last receiver throws  "request unhandled".
     *
     * The current handler gets asked if it wants to handle the request. If
     * not, the request is forwarded to its successor.
     */
	protected void receiveRequest(Handler handler, Request request) {  
		if (handler.acceptRequest(request)) {
		    handler.handleRequest(request);
		} else {
  		    Handler successor = getSuccessor(handler);         
		    if (successor == null) {
		    	System.out.println("REQUEST UNHANDLED (END OF CHAIN REACHED)\n");
				//throw new ChainOfResponsibilityException("request unhandled (end of chain reached)\n");
				// This is one way to deal with unhandled requests.
		    } else {
   	    	    receiveRequest(successor, request);
   	    	}
		}
	}  

    /**
     * Adds a successor to a Handler.  
     *
     * @param handler the handler to add a new successor to
     * @param successor the new successor to attach
     */
	public void setSuccessor(Handler handler, Handler successor) {
		successors.put(handler, successor);
	}  

    /**
     * Returns the successor of a handler.  
     *
     * @param handler the handler in question
     * @return the successor of the handler
     */     
	protected Handler getSuccessor(Handler handler) {
		return successors.get(handler);
	}
}