package chain_of_responsibility.hk;

import org.objectteams.Team;

/**
 * Implements a GUI-motivated example for the Chain Of Responsibility design
 * pattern.
 * 
 * Intent: Avoid coupling the sender of a request to its receiver by giving
 * more than one object a chance to handle the request. Chain the receiving
 * objects and pass the request along the chain until an object handles it.
 * 
 * Participatng objects are a Frame, a Panel, and Button
 * 
 * A click on the button triggers an event (request) that gets passed along the
 * widget hierarchy (button -> panel -> frame).
 * 
 * Clicking the button will start a request, that gets passed on along the
 * following chain: button, panel, frame. Depending on whether the ALT, SHIFT,
 * or CTRL keys are pressed during the button click, a different object in the
 * chain will handle the request:
 * 
 * <ol>
 * <li>If the SHIFT key is pressed, Button will handle the request
 * <li>If the CTRL key is pressed, Panel will handle the request
 * <li>If the ALT key is pressed, Frame will handle the request
 * <li>If no keys are pressed, the request will not be handled and an exception
 * will be raised.
 * </ol>
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {
	public static void main(String[] args) {
		Button button = new Button(
				"Click me to see the pattern in action! Use <SHIFT>, <CTRL>, and <ALT> during clicks to see different behavior");
		Frame frame = new Frame("OT/J Chain of Responsibility pattern example");
		Panel panel = new Panel();

		frame.getContentPane().add(panel);
		panel.add(button);

		frame.pack();
		frame.setVisible(true);

		ClickChainTeam ccTeam = new ClickChainTeam();
		ccTeam.activate(Team.ALL_THREADS);
		ccTeam.setSucessor(button, panel);
		ccTeam.setSucessor(panel, frame);
	}
}
