package flyweight;

import java.util.Hashtable;

/**
 * Implements the abstract team for the Flyweight design pattern. Included is
 * the general creation-on-demand logic. Concrete teams are used to define the
 * actual FlyweightFactories.
 * 
 * Concrete teams need only to assign the flyweight roles to base classes and to
 * overwrite the createFlyweight(Object) method.
 * 
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public abstract team class FlyweightProtocol {

	/**
	 * stores Objects representing the existing Flyweights by key
	 */
	protected Hashtable<Object, Flyweight> flyweights = new Hashtable<Object, Flyweight>();

	/**
	 * defines the Flyweight role.
	 */
	protected abstract class Flyweight implements ILowerable {
	}

	/**
	 * Creates a Flyweight for a given key. This method is called by
	 * getFlyweight(Object) if the flyweight does not already
	 * exist.
	 * 
	 * @param key	the key identifying the particular flyweight
	 * @return	the Flyweight representing the key
	 */
	protected abstract Flyweight createFlyweight(Object key);

	/**
	 * Returns an Object representing the Flyweight for a particular key.
	 * Object is returned since this method is to be called from outside the
	 * team. If the appropriate Flyweight does not yet exist, it is
	 * created on demand.
	 * 
	 * @param key
	 *            the key identifying the particular Flyweight
	 * @return the Flyweight representing the key
	 */
	public Object getFlyweight(Object key) {
		if (flyweights.containsKey(key))
			return (flyweights.get(key)).lower();
		else {
			Flyweight flyweight = createFlyweight(key);
			flyweights.put(key, flyweight);
			return flyweight.lower();
		}
	}
}
