package interpreter.hk;

import java.util.Hashtable;

/**
 * This team represents the variableContext that existed in the original example
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 **/
public abstract team class InterpreterAbstTeam {

	/**
	 * stores the mapping between variable names and values
	 */
	protected Hashtable<String, Boolean> assignments = new Hashtable<String, Boolean>();

	public abstract class BooleanExpression {
		/**
		 * Evaluates this Expression in the given VariableContext
		 * 
		 * @param c
		 *            the Context to evaluate the Expression in
		 * @return the boolean value of the Expression
		 */
		public abstract boolean evaluate();
	
		/**
		 * Replaces a variable with an Expression
		 * 
		 * @param name
		 *            the name of the variable
		 * @param exp
		 *            the Expression to replace the variable
		 * @return a copy of this Expression with the variable replaced
		 */
		public abstract BooleanExpression replace(String name,
				BooleanExpression exp);
	
		/**
		 * Copies this Expression
		 * 
		 * @return the copied Expression
		 */
		public abstract BooleanExpression copy();
	}

	public class VariableExpression extends BooleanExpression {
	
		/**
		 * the name of the variable this object represents
		 */
		protected String name = null;
	
		/**
		 * Creates a new VariableExpression with the given name
		 * 
		 * @param name
		 *            the name of the new variable
		 */
		public VariableExpression(String name) {
			this.name = name;
		}
	
		/**
		 * Accessor for the variable's name
		 * 
		 * @return the name of the variable
		 */
		public String getName() {
			return name;
		}

		/**
		 * Evaluates this Expression in the given Context
		 * 
		 * @param c
		 *            the context to evaluate the Expression in
		 * @return the boolean value of the Expression
		 */
		public boolean evaluate() {
			return lookup(name);
		}
	
		/**
		 * Replaces a variable with an Expression
		 * 
		 * @param name
		 *            the name of the variable
		 * @param exp
		 *            the Expression to replace the variable
		 * @return a copy of this Expression with the variable replaced
		 */
		public BooleanExpression replace(String name, BooleanExpression exp) {
			if (name.equals(this.name))
				return exp.copy();
			else
				return new VariableExpression(this.name);
		}
	
		/**
		 * Copies this Expression
		 * 
		 * @return the copied Expression
		 */
		public BooleanExpression copy() {
			return new VariableExpression(name);
		}
	}

	public class BooleanConstant extends BooleanExpression {
		/**
		 * the value of this constant
		 */
		protected boolean value;
	
		/**
		 * Creates a new constant with the given value
		 * 
		 * @param value
		 *            the value this constant should represent
		 */
		public BooleanConstant(boolean value) {
			this.value = value;
		}
	
		/**
		 * Evaluates this Expression in the given Context.
		 * 
		 * @param c
		 *            the Context to evaluate the Expression in
		 * @return the boolean value of the Expression
		 */
		public boolean evaluate() {
			return value;
		}

		/**
		 * Replaces a variable with an Expression. Has no effect on
		 * constants.
		 * 
		 * @param name
		 *            the name of the variable
		 * @param exp
		 *            the Expression to replace the variable
		 * @return the unchanged constant
		 */
		public BooleanExpression replace(String name, BooleanExpression exp) {
			return this;
		}
	
		/**
		 * Copies this Expression
		 * 
		 * @return the copied Expression
		 */
		public BooleanExpression copy() {
			return new BooleanConstant(value);
		}
	}

	/**
	 * Returns the current value for a variable
	 * 
	 * @param name the name of the variable
	 * @return the value of the variable
	 */
	public boolean lookup(String name) {
		Boolean value = (Boolean) assignments.get(name);
		if (value == null) {
			throw new RuntimeException("No variable \"" + name + "\" known.");
		}
		return value.booleanValue();
	}

	/**
	 * Assigns a boolean value to a VariableExpression
	 * 
	 * @param varExp the variable Expression to assign a value to
	 * @param bool the boolean value to assign
	 */
	public void assign(VariableExpression varExp, boolean bool) {
		assignments.put(varExp.getName(), new Boolean(bool));
	}
}
