package interpreter.hk;

/**
 * Implements the driver for the Intepreter design pattern example.<p> 
 *
 * Intent: Given a language, define a representation for its grammar along
 * with an interpreter that uses the representation to interpret sentences
 * in the language.
 *
 * This example implements an interpreter for a language of boolean 
 * expressions. As a sample expression, "((true & x) | (y & !x))" is
 * interpreted for all possible boolean values for x and y. After that, 
 * y is replaced by another expression and the whole expression is
 * evaluated again. 
 *
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {  

    /** 
     * Assigns boolean values to two <code>VariableExpression</code>s 
     * and evaluates an expression in the given context.
     *
     * @param context the context to evaluate the expression in
     * @param x a boolean variable expression
     * @param xValue the value to assign to x
     * @param y another boolean variable expression
     * @param yValue the value to assign to y
     * @param exp the expression to evaluate
     */
	private static void assignAndEvaluate(
			final InterpreterTeam context, 
			VariableExpression<@context> x, 
			boolean xValue,
			VariableExpression<@context> y, 
			boolean yValue,			
			BooleanExpression<@context> exp) {
		context.assign(x, xValue);
		context.assign(y, yValue);
		boolean result = exp.evaluate();
		System.out.println("The OTJ result for (x="+xValue+", y="+yValue+") is: "+result);
	}

    /**
     * Implements the driver for the Intepreter design pattern example.<p> 
     *
     * @param command-line parameters, unused.
     */
	public static void main(String[] args) {
		final InterpreterTeam interpreter = new InterpreterTeam();
		
		BooleanExpression<@interpreter> exp = null;
		
		VariableExpression<@interpreter> x = new VariableExpression<@interpreter>("X");
		VariableExpression<@interpreter> y = new VariableExpression<@interpreter>("Y");		

//		final InterpreterTeam otherInterpreter = new InterpreterTeam();
//		BooleanExpression<@otherInterpreter> exp1 = new NotExpression<@interpreter>(x);

		exp = new OrExpression<@interpreter>(new AndExpression<@interpreter>(
				new BooleanConstant<@interpreter>(true), x),
				new AndExpression<@interpreter>(y, new NotExpression<@interpreter>(x)));
		
		//exp = new NotExpression<@interpreter>(x); 
		
		System.out.println("Testing Expression: ((true & x) | (y & !x))");			 

		assignAndEvaluate(interpreter, x, false, y, false,  exp);
		assignAndEvaluate(interpreter, x, false, y, true,  exp);
		assignAndEvaluate(interpreter, x, true,  y, false, exp);
		assignAndEvaluate(interpreter, x, true,  y, true,  exp);

		VariableExpression<@interpreter> z = new VariableExpression<@interpreter>("Z");
		NotExpression<@interpreter>   notZ = new NotExpression<@interpreter>(z);

		BooleanExpression<@interpreter> replacement = exp.replace("Y", notZ);
		interpreter.assign(z, false);
		boolean result = replacement.evaluate();

		System.out.println("The OTJ result for the replacement is: "+result);
	}
}
