package observer.hk;

import java.awt.Color;

/**
 * Implements the driver for the Observer design pattern example.
 * 
 * Intent: Define a one-to-many dependency between objects so that when one
 * object changes state, all its dependents are notified and updated
 * automatically
 * 
 * Participating objects are Point p and Screen s1,
 * s2, s3, s4, and s5.
 * 
 * Three different kinds of observing relationships are realized:
 * <UL>
 * <LI> Screen s1 and s2 observe color changes of Point
 *       p.
 * <LI> Screen s3 and s4 observe coordinate changes of 
 *      Point
 * p.
 * <LI> Screen s5 observes the display(String) methods
 * of Screen s2 and s4.
 * 
 * Every time an event of interest occurs, the observing Screen
 * prints an appropriate message to stdout.
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {

	/**
	 * Implements the driver for the Observer example. It creates five
	 * Screen objects and one Point object and sets
	 * the appropriate observing relationships (see above). After the setup, the
	 * color of the point is changed, then it's x-coordinate.
	 * 
	 * The following results should be expected:
	 * <OL>
	 * <LI>The color change should trigger s1 and s2 to each print an
	 * appropriate message.
	 * <LI>s2's message should trigger it's observer s5 to print a message.
	 * <LI>The coordinate change should trigger s3 and s4.
	 * <LI>s4's message should trigger it's observer s5 again.
	 */
	public static void main(String argv[]) {

		Point p = new Point(5, 5, Color.blue);

		System.out.println("Creating Screen s1,s2,s3,s4,s5 and Point p");

		Screen s1 = new Screen("s1");
		Screen s2 = new Screen("s2");

		Screen s3 = new Screen("s3");
		Screen s4 = new Screen("s4");

		Screen s5 = new Screen("s5");

		ColorObserver colorO = new ColorObserver();
		colorO.activate();
		CoordinatesObserver coordO = new CoordinatesObserver();
		coordO.activate();
		ScreenObserver screenO = new ScreenObserver();
		screenO.activate();

		System.out.println("Creating observing relationships:");
		System.out.println("- s1 and s2 observe color changes to p");
		System.out.println("- s3 and s4 observe coordinate changes to p");
		System.out.println("- s5 observes s2's and s4's display() method");

		colorO.addObserver(s1, p);
		colorO.addObserver(s2, p);

		coordO.addObserver(s3, p);
		coordO.addObserver(s4, p);

		screenO.addObserver(s5, s2);
		screenO.addObserver(s5, s4);

		System.out.println("Changing p's color:");

		p.setColor(Color.red);

		System.out.println("Changing p's x-coordinate:");

		p.setX(4);

		System.out.println("done.");
	}
}
