package singleton.hk;

/**
 * Implements the driver for the Singleton design pattern example. Intent:
 * Ensure that a class has only one instance and provide a global point of
 * access to it. Participating objects are PrinterSingleton printer1, printer2,
 * printer3 and PrinterSubclass ps1, ps2, ps3. Three different objects of both
 * PrinterSingleton and PrinterSubclass are instantiated and compared. This
 * Implementation treats the singleton property as a non-inherited property.
 * This meant that Singleton classes can still be subclassed and these
 * subclasses can access the Singleton constructor normally. Making the
 * singleton property non-inherited requires to make the
 * constructor of the Singleton protected (so subtypes can still invoke
 * super(..) in their constructor).
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {

	/**
	 * the three object references to instances of the Singleton class.
	 */
	private static PrinterSingleton printer1, printer2, printer3;

	/**
	 * Implements the first test case. Creates 3 references to the Singleton by
	 * (a) using the regular constructor and then by
	 * (b) using a factory method.
	 * (a) should fail (but does not, since this Window4Mediator class is in the same
	 * package and can access the protected constructor), (b) should create
	 * three identical objects.
	 */
	private static void test1() {
		System.out.println("\nTest 1a: Try to call regular constructor. "
				+ "This should fail.");
		printer1 = new PrinterSingleton();
		printer2 = new PrinterSingleton();
		printer3 = new PrinterSingleton();
		printer1.print();
		printer2.print();
		printer3.print();
		System.out.println("\t=> OO Problem: Classes in the same package can "
				+ "access the protected constructor.");
		System.out.println("\nTest 1b: Using instance() instead. This should "
				+ "create three identical objects.");
		printer1 = PrinterSingleton.instance();
		printer2 = PrinterSingleton.instance();
		printer3 = PrinterSingleton.instance();
		printer1.print();
		printer2.print();
		printer3.print();
	}

	/**
	 * Implements the second test case. Tests if the 3 objects from test 1 are
	 * in fact identical
	 */
	private static void test2() {
		System.out.println("\nTest 2: All three objects should be identical");
		System.out.print("\tThey are ");
		if ((printer1 == printer2) && (printer1 == printer3)) {
			System.out.println("identical");
		} else {
			System.out.println("not identical");
		}
	}

	/**
	 * Implements the third test case. Creates 3 instances of the Singleton 's
	 * subclass. These objects should be different.
	 */
	private static void test3() {
		System.out.println("\nTest 3: Ensuring that subclasses can access the"
				+ " constructor");
		System.out.println("        (All three outputs should be different)");
		printer1 = new PrinterSubclass();
		printer2 = new PrinterSubclass();
		printer3 = new PrinterSubclass();
		printer1.print();
		printer2.print();
		printer3.print();
	}

	/**
	 * This is the driver for the Singleton case. It performs three tests:
	 * Creates 3 references to the Singleton by
	 * (a) using the regular constructor and then by
	 * (b) using a factory method.
	 * (a) should fail, (b) should create three identical objects.
	 * Tests if the above 3 objects are in fact identical.
	 * Creates 3 instances of a Singleton's subclass. These objects should
	 * be different.
	 */
	public static void main(String[] args) {
		System.out.println("Testing SINGLETON pattern (java) ...");
		test1();
		test2();
		test3();
		System.out.println("\n... done.");
	}
}
